#pragma once

// all the nodes
#include "node/AprilTag.hpp"
#include "node/ColorCamera.hpp"
#include "node/DetectionNetwork.hpp"
#include "node/DetectionParser.hpp"
#include "node/EdgeDetector.hpp"
#include "node/FeatureTracker.hpp"
#include "node/IMU.hpp"
#include "node/ImageManip.hpp"
#include "node/MonoCamera.hpp"
#include "node/NeuralNetwork.hpp"
#include "node/ObjectTracker.hpp"
#include "node/SPIIn.hpp"
#include "node/SPIOut.hpp"
#include "node/Script.hpp"
#include "node/SpatialDetectionNetwork.hpp"
#include "node/SpatialLocationCalculator.hpp"
#include "node/StereoDepth.hpp"
#include "node/SystemLogger.hpp"
#include "node/VideoEncoder.hpp"
#include "node/XLinkIn.hpp"
#include "node/XLinkOut.hpp"