#pragma once

// Includes common classes needed to start the development

// Include initialization utility
#include "utility/Initialization.hpp"

// Include some common device classes
#include "device/CalibrationHandler.hpp"
#include "device/Device.hpp"
#include "device/DeviceBootloader.hpp"

// Include Pipeline
#include "pipeline/Pipeline.hpp"

// Include common nodes
#include "pipeline/nodes.hpp"

// Include common datatypes
#include "pipeline/datatypes.hpp"

// namespace dai {
// namespace{
// bool initializeForce = [](){initialize();};
// } // namespace
// }
