/*
 * Generated by CMake build tool.
 */
#pragma once

// This build supports OpenCV integration?
/* #undef DEPTHAI_HAVE_OPENCV_SUPPORT */

// Build specific settings overwrite
#ifdef DEPTHAI_TARGET_CORE
#ifndef DEPTHAI_TARGET_OPENCV
#undef DEPTHAI_HAVE_OPENCV_SUPPORT
#endif
#endif
