/*
 * Generated by CMake build tool.
 * To modify version, open CMakeLists.txt and bump the version in project() function
 */
#pragma once

namespace dai
{
namespace build
{
    constexpr static const char* VERSION = "2.17.1";
    constexpr static const int VERSION_MAJOR = 2;
    constexpr static const int VERSION_MINOR = 17;
    constexpr static const int VERSION_PATCH = 1;

    constexpr static const char* COMMIT = "c455d09d330bb318211d57690fa9188bfd499640";
    constexpr static const char* COMMIT_DATETIME = "2022-07-19 00:24:11 +0300";
    constexpr static const char* BUILD_DATETIME = "2022-07-19 00:07:58 +0000";
} // namespace build
} // namespace dai
