#pragma once

// all the data types
#include "datatype/ADatatype.hpp"
#include "datatype/AprilTagConfig.hpp"
#include "datatype/AprilTags.hpp"
#include "datatype/Buffer.hpp"
#include "datatype/CameraControl.hpp"
#include "datatype/FeatureTrackerConfig.hpp"
#include "datatype/IMUData.hpp"
#include "datatype/ImageManipConfig.hpp"
#include "datatype/ImgDetections.hpp"
#include "datatype/ImgFrame.hpp"
#include "datatype/NNData.hpp"
#include "datatype/SpatialImgDetections.hpp"
#include "datatype/SpatialLocationCalculatorConfig.hpp"
#include "datatype/SpatialLocationCalculatorData.hpp"
#include "datatype/SystemInformation.hpp"
#include "datatype/TrackedFeatures.hpp"
#include "datatype/Tracklets.hpp"